## START: Set by rpmautospec
## (rpmautospec version 0.2.5)
%define autorelease(e:s:pb:) %{?-p:0.}%{lua:
    release_number = 1;
    base_release_number = tonumber(rpm.expand("%{?-b*}%{!?-b:1}"));
    print(release_number + base_release_number - 1);
}%{?-e:.%{-e*}}%{?-s:.%{-s*}}%{?dist}
## END: Set by rpmautospec

%global srcname typer
%global forgeurl https://github.com/tiangolo/%{srcname}
%global tag %{version}

# As of 2021-08-11, mkdocs and many of its dependencies are orphaned.
%bcond_with mkdocs

Name:           python-%{srcname}
Version:        0.4.0
%forgemeta
Release:        %autorelease
Summary:        Build great CLIs; easy to code; based on Python type hints

License:        MIT
URL:            https://%{srcname}.tiangolo.com/
Source0:        %{forgesource}
BuildArch:      noarch

BuildRequires:  python3-devel
BuildRequires:  pyproject-rpm-macros

# Documentation
BuildRequires:  web-assets-devel
%if %{with mkdocs}
# Would have been pulled in indirectly via mkdocs-material, but we have patched
# that out:
BuildRequires:  python3dist(pymdown-extensions)
%endif

%global common_description %{expand:
Typer is a library for building CLI applications that users will love using and
developers will love creating. Based on Python 3.6+ type hints.

The key features are:

  • Intuitive to write: Great editor support. Completion everywhere. Less time
    debugging. Designed to be easy to use and learn. Less time reading docs.
  • Easy to use: It’s easy to use for the final users. Automatic help, and
    automatic completion for all shells.
  • Short: Minimize code duplication. Multiple features from each parameter
    declaration. Fewer bugs.
  • Start simple: The simplest example adds only 2 lines of code to your app: 1
    import, 1 function call.
  • Grow large: Grow in complexity as much as you want, create arbitrarily
    complex trees of commands and groups of subcommands, with options and
    arguments.

Typer is FastAPI’s little sibling.

And it’s intended to be the FastAPI of CLIs.}

%description %{common_description}


%package -n     python3-%{srcname}
Summary:        %{summary}

%description -n python3-%{srcname} %{common_description}


%pyproject_extras_subpkg -n python3-%{srcname} all


%package        doc
Summary:        Documentation for Typer

Requires:       js-termynal

%description    doc
Documentation for Typer.


%prep
%forgeautosetup
# We must loosen up some strict dependencies to accommodate the versions that
# are packaged. This is always a little risky, but fortunately it affects
# build, documentation, and test dependencies rather than runtime ones.
#
#   1. We have flit 3.x in Fedora 34 and later, so we must try to use it.
#   2. Filter out formatter/linter tools that we will not actually use from the
#      test extra requirements; we would not bother, except that they have very
#      strict version pins that we will not be able to satisfy.
#   3. Loosen certain test version pins as well; we must work with what we
#      have. We allow some to be newer.
sed -r -i \
    -e 's/(flit_core[[:blank:]]*>=2[^,]*),<3\b/\1/' \
    -e '/\b(mypy|black|isort)[[:blank:]]*[>=]=/d' \
    -e 's/\b(pytest(-xdist)?)([[:blank:]]*>=[^,]*),[^"]+/\1\3/' \
    pyproject.toml

# Remove bundled js-termynal 0.0.1:
find docs -type f -name 'termynal.*' |
  while read -r fn
  do
    echo '' > "${fn}"  # make it an empty file
  done
# In %%install, we will restore this functionality by symlinking the system
# copy.

%if %{with mkdocs}
# Drop dependency on mkdocs-material. See
# https://bugzilla.redhat.com/show_bug.cgi?id=1960274#c3. The result doesn’t
# look as nice but is still usable.
sed -r -i 's/\bmaterial\b/mkdocs/' mkdocs.yml
sed -r -i '/mkdocs-material/d' pyproject.toml
# Allow certain documentation dependencies to be newer.
sed -r -i \
    -e 's/\b(markdown-include)([[:blank:]]*>=[^,]*),[^"]+/\1\3/' \
    pyproject.toml
%endif


%generate_buildrequires
%pyproject_buildrequires -x all,test%{?with_mkdocs:,doc}

%build
%pyproject_wheel
%if %{with mkdocs}
mkdocs build
mv site html
%endif


%install
%pyproject_install
%pyproject_save_files %{srcname}

# Replace truncated js-termynal files with absolute symbolic links
find docs -type f -name 'termynal.*' |
  while read -r fn
  do
    ln -s -f "%{_jsdir}/$(basename "${fn}")" "${fn}"
  done


%check
# See scripts/test.sh. We do not run the linters (scripts/lint.sh, i.e.,
# mypy/black/isort).
./scripts/test-files.sh
# Shell completion tests need us to be running under a supported shell, i.e.
# bash rather than sh. Unfortunately, shell detection with shellingham is so
# thorough we cannot fool it by any combination of:
#  - export SHELL=/bin/bash
#  - bash -c '%%pytest'
#  - %%check -p /bin/bash
# so we must simply skip the affected tests.
%pytest -k 'not test_show_completion and not test_install_completion'


%files -n python3-%{srcname} -f %{pyproject_files}
%license LICENSE


%files doc
%license LICENSE
%doc CONTRIBUTING.md
%doc README.md
%if %{with mkdocs}
%doc html
%else
%doc docs
%doc docs_src
%endif


%changelog
* Tue Aug 31 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.4.0-1
- Update to 0.4.0 (fix RHBZ#1977513)

* Wed Jun 30 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 0.3.2-4
- Remove reference to bug for out-of-date mkdocs-material

* Thu Jun 17 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 0.3.2-3
- Drop documentation dependency on mkdocs-material (RHBZ#1960274, comment 3)

* Wed May 26 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 0.3.2-2
- Unbundle js-termynal

* Tue May 25 2021 Benjamin A. Beasley <code@musicinmybrain.net> - 0.3.2-1
- Initial package

